function [fs,rigidez,newparam] = restoringforce(u,uold,v,vold,fsold,param)
% Elastic plastic hysteresis model

if nargout==1 %Initialize hysteresis loop
   fs.xu = 0;   fs.yu = 0;
   fs.k = u;    fs.r = uold;   fs.inter_y = v;
   return;
elseif nargout<=3
   xu = param.xu;    yu = param.yu;  
   k = param.k;      r = param.r;      inter_y = param.inter_y;
else
   error('Invalid number of output parameters');
end;

k_r = k+r;

fs = yu + k_r*(u-xu);

rigidez = k_r;

if (v>0)
   fyp = r*u + inter_y;
   if (fs>fyp)
      fs = fyp;
      xu = (inter_y + k_r*xu -yu)/k; % 1
      yu = r*xu + inter_y;
      rigidez = r;
   end;
   if (vold<0) % && (v>0)
      if abs(fsold-r*uold+inter_y) < 1e-3
         xu = uold;
         yu = fsold;
      else
         xu = (-inter_y + k_r*u - fs)/k; % 4
         yu = r*xu - inter_y;
      end;
      fs = yu + k_r*(u-xu);
   end;
else %(v<0)
   fyn = r*u - inter_y;
   if (fs<fyn)
      fs = fyn;
      xu = (-inter_y + k_r*xu - yu)/k; %3
      yu = r*xu - inter_y;
      rigidez = r;
   end;
   if (vold>0) % && (v<0)
      if abs(fsold-r*uold-inter_y) < 1e-3
         xu = uold;
         yu = fsold;
      else
         xu = (inter_y + k_r*u - fs)/k; % 2
         yu = r*xu + inter_y;
      end;
      fs = yu + k_r*(u-xu);
   end
end;   

newparam.xu = xu;
newparam.yu = yu;
newparam.k = k;
newparam.r = r;
newparam.inter_y = inter_y;

return;
